var Maze, MersenneTwister,
  __hasProp = {}.hasOwnProperty,
  __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

Maze = (function() {

  function Maze(width, height, algorithm, options) {
    this.width = width;
    this.height = height;
    if (options == null) {
      options = {};
    }
    this.grid = new Maze.Grid(this.width, this.height);
    this.rand = options.rng || new MersenneTwister(options.seed);
    this.isWeave = options.weave;
    if (this.rand.randomElement == null) {
      this.rand.randomElement = function(list) {
        return list[this.nextInteger(list.length)];
      };
      this.rand.removeRandomElement = function(list) {
        var results;
        results = list.splice(this.nextInteger(list.length), 1);
        if (results) {
          return results[0];
        }
      };
      this.rand.randomizeList = function(list) {
        var i, j, _ref;
        i = list.length - 1;
        while (i > 0) {
          j = this.nextInteger(i + 1);
          _ref = [list[j], list[i]], list[i] = _ref[0], list[j] = _ref[1];
          i--;
        }
        return list;
      };
      this.rand.randomDirections = function() {
        return this.randomizeList(Maze.Direction.List.slice(0));
      };
    }
    this.algorithm = new algorithm(this, options);
  }

  Maze.prototype.onUpdate = function(fn) {
    return this.algorithm.onUpdate(fn);
  };

  Maze.prototype.onEvent = function(fn) {
    return this.algorithm.onEvent(fn);
  };

  Maze.prototype.generate = function() {
    var _results;
    _results = [];
    while (true) {
      if (!this.step()) {
        break;
      } else {
        _results.push(void 0);
      }
    }
    return _results;
  };

  Maze.prototype.step = function() {
    return this.algorithm.step();
  };

  Maze.prototype.isEast = function(x, y) {
    return this.grid.isMarked(x, y, Maze.Direction.E);
  };

  Maze.prototype.isWest = function(x, y) {
    return this.grid.isMarked(x, y, Maze.Direction.W);
  };

  Maze.prototype.isNorth = function(x, y) {
    return this.grid.isMarked(x, y, Maze.Direction.N);
  };

  Maze.prototype.isSouth = function(x, y) {
    return this.grid.isMarked(x, y, Maze.Direction.S);
  };

  Maze.prototype.isUnder = function(x, y) {
    return this.grid.isMarked(x, y, Maze.Direction.U);
  };

  Maze.prototype.isValid = function(x, y) {
    return (0 <= x && x < this.width) && (0 <= y && y < this.height);
  };

  Maze.prototype.carve = function(x, y, dir) {
    return this.grid.mark(x, y, dir);
  };

  Maze.prototype.uncarve = function(x, y, dir) {
    return this.grid.clear(x, y, dir);
  };

  Maze.prototype.isSet = function(x, y, dir) {
    return this.grid.isMarked(x, y, dir);
  };

  Maze.prototype.isBlank = function(x, y) {
    return this.grid.at(x, y) === 0;
  };

  Maze.prototype.isPerpendicular = function(x, y, dir) {
    return (this.grid.at(x, y) & Maze.Direction.Mask) === Maze.Direction.cross[dir];
  };

  return Maze;

})();

Maze.Algorithms = {};

Maze.Algorithm = (function() {

  function Algorithm(maze, options) {
    this.maze = maze;
    if (options == null) {
      options = {};
    }
    this.updateCallback = function(maze, x, y) {};
    this.eventCallback = function(maze, x, y) {};
    this.rand = this.maze.rand;
  }

  Algorithm.prototype.onUpdate = function(fn) {
    return this.updateCallback = fn;
  };

  Algorithm.prototype.onEvent = function(fn) {
    return this.eventCallback = fn;
  };

  Algorithm.prototype.updateAt = function(x, y) {
    return this.updateCallback(this.maze, x, y);
  };

  Algorithm.prototype.eventAt = function(x, y) {
    return this.eventCallback(this.maze, x, y);
  };

  Algorithm.prototype.canWeave = function(dir, thruX, thruY) {
    var nx, ny;
    if (this.maze.isWeave && this.maze.isPerpendicular(thruX, thruY, dir)) {
      nx = thruX + Maze.Direction.dx[dir];
      ny = thruY + Maze.Direction.dy[dir];
      return this.maze.isValid(nx, ny) && this.maze.isBlank(nx, ny);
    }
  };

  Algorithm.prototype.performThruWeave = function(thruX, thruY) {
    if (this.rand.nextBoolean()) {
      return this.maze.carve(thruX, thruY, Maze.Direction.U);
    } else if (this.maze.isNorth(thruX, thruY)) {
      this.maze.uncarve(thruX, thruY, Maze.Direction.N | Maze.Direction.S);
      return this.maze.carve(thruX, thruY, Maze.Direction.E | Maze.Direction.W | Maze.Direction.U);
    } else {
      this.maze.uncarve(thruX, thruY, Maze.Direction.E | Maze.Direction.W);
      return this.maze.carve(thruX, thruY, Maze.Direction.N | Maze.Direction.S | Maze.Direction.U);
    }
  };

  Algorithm.prototype.performWeave = function(dir, fromX, fromY, callback) {
    var thruX, thruY, toX, toY;
    thruX = fromX + Maze.Direction.dx[dir];
    thruY = fromY + Maze.Direction.dy[dir];
    toX = thruX + Maze.Direction.dx[dir];
    toY = thruY + Maze.Direction.dy[dir];
    this.maze.carve(fromX, fromY, dir);
    this.maze.carve(toX, toY, Maze.Direction.opposite[dir]);
    this.performThruWeave(thruX, thruY);
    if (callback) {
      callback(toX, toY);
    }
    this.updateAt(fromX, fromY);
    this.updateAt(thruX, thruY);
    return this.updateAt(toX, toY);
  };

  return Algorithm;

})();

Maze.Direction = {
  N: 0x01,
  S: 0x02,
  E: 0x04,
  W: 0x08,
  U: 0x10,
  Mask: 0x01 | 0x02 | 0x04 | 0x08 | 0x10,
  List: [1, 2, 4, 8],
  dx: {
    1: 0,
    2: 0,
    4: 1,
    8: -1
  },
  dy: {
    1: -1,
    2: 1,
    4: 0,
    8: 0
  },
  opposite: {
    1: 2,
    2: 1,
    4: 8,
    8: 4
  },
  cross: {
    1: 4 | 8,
    2: 4 | 8,
    4: 1 | 2,
    8: 1 | 2
  }
};

Maze.Grid = (function() {

  function Grid(width, height) {
    var x, y;
    this.width = width;
    this.height = height;
    this.data = (function() {
      var _i, _ref, _results;
      _results = [];
      for (y = _i = 1, _ref = this.height; 1 <= _ref ? _i <= _ref : _i >= _ref; y = 1 <= _ref ? ++_i : --_i) {
        _results.push((function() {
          var _j, _ref1, _results1;
          _results1 = [];
          for (x = _j = 1, _ref1 = this.width; 1 <= _ref1 ? _j <= _ref1 : _j >= _ref1; x = 1 <= _ref1 ? ++_j : --_j) {
            _results1.push(0);
          }
          return _results1;
        }).call(this));
      }
      return _results;
    }).call(this);
  }

  Grid.prototype.at = function(x, y) {
    return this.data[y][x];
  };

  Grid.prototype.mark = function(x, y, bits) {
    return this.data[y][x] |= bits;
  };

  Grid.prototype.clear = function(x, y, bits) {
    return this.data[y][x] &= ~bits;
  };

  Grid.prototype.isMarked = function(x, y, bits) {
    return (this.data[y][x] & bits) === bits;
  };

  return Grid;

})();

/*
This implementation of the Mersenne Twister is a port of the JavaScript
version by Y. Okada. The JavaScript version was itself a port of a
C implementation, by Takuji Nishimura and Makoto Matsumoto.

CoffeeScript port by: Jamis Buck <jamis@jamisbuck.org>
License: Public domain, baby. Knock yourself out.

The original CoffeeScript sources are always available on GitHub:
http://github.com/jamis/csmazes
*/


MersenneTwister = (function() {

  MersenneTwister.prototype.N = 624;

  MersenneTwister.prototype.M = 397;

  MersenneTwister.prototype.MATRIX_A = 0x9908b0df;

  MersenneTwister.prototype.UPPER_MASK = 0x80000000;

  MersenneTwister.prototype.LOWER_MASK = 0x7fffffff;

  function MersenneTwister(seed) {
    this.mt = new Array(this.N);
    this.setSeed(seed);
  }

  MersenneTwister.prototype.unsigned32 = function(n1) {
    if (n1 < 0) {
      return (n1 ^ this.UPPER_MASK) + this.UPPER_MASK;
    } else {
      return n1;
    }
  };

  MersenneTwister.prototype.subtraction32 = function(n1, n2) {
    if (n1 < n2) {
      return this.unsigned32((0x100000000 - (n2 - n1)) % 0xffffffff);
    } else {
      return n1 - n2;
    }
  };

  MersenneTwister.prototype.addition32 = function(n1, n2) {
    return this.unsigned32((n1 + n2) & 0xffffffff);
  };

  MersenneTwister.prototype.multiplication32 = function(n1, n2) {
    var i, sum, _i;
    sum = 0;
    for (i = _i = 0; _i < 32; i = ++_i) {
      if ((n1 >>> i) & 0x1) {
        sum = this.addition32(sum, this.unsigned32(n2 << i));
      }
    }
    return sum;
  };

  MersenneTwister.prototype.setSeed = function(seed) {
    if (!seed || typeof seed === "number") {
      return this.seedWithInteger(seed);
    } else {
      return this.seedWithArray(seed);
    }
  };

  MersenneTwister.prototype.defaultSeed = function() {
    var currentDate;
    currentDate = new Date();
    return currentDate.getMinutes() * 60000 + currentDate.getSeconds() * 1000 + currentDate.getMilliseconds();
  };

  MersenneTwister.prototype.seedWithInteger = function(seed) {
    var _results;
    this.seed = seed != null ? seed : this.defaultSeed();
    this.mt[0] = this.unsigned32(this.seed & 0xffffffff);
    this.mti = 1;
    _results = [];
    while (this.mti < this.N) {
      this.mt[this.mti] = this.addition32(this.multiplication32(1812433253, this.unsigned32(this.mt[this.mti - 1] ^ (this.mt[this.mti - 1] >>> 30))), this.mti);
      this.mti[this.mti] = this.unsigned32(this.mt[this.mti] & 0xffffffff);
      _results.push(this.mti++);
    }
    return _results;
  };

  MersenneTwister.prototype.seedWithArray = function(key) {
    var i, j, k, _m;
    this.seedWithInteger(19650218);
    i = 1;
    j = 0;
    k = this.N > key.length ? this.N : key.length;
    while (k > 0) {
      _m = this.multiplication32(this.unsigned32(this.mt[i - 1] ^ (this.mt[i - 1] >>> 30)), 1664525);
      this.mt[i] = this.addition32(this.addition32(this.unsigned32(this.mt[i] ^ _m), key[j]), j);
      this.mt[i] = this.unsigned32(this.mt[i] & 0xffffffff);
      i++;
      j++;
      if (i >= this.N) {
        this.mt[0] = this.mt[this.N - 1];
        i = 1;
      }
      if (j >= key.length) {
        j = 0;
      }
      k--;
    }
    k = this.N - 1;
    while (k > 0) {
      this.mt[i] = this.subtraction32(this.unsigned32(this.mt[i] ^ this.multiplication32(this.unsigned32(this.mt[i - 1] ^ (this.mt[i - 1] >>> 30)), 1566083941)), i);
      this.mt[i] = this.unsigned32(this.mt[i] & 0xffffffff);
      i++;
      if (i >= this.N) {
        this.mt[0] = this.mt[this.N - 1];
        i = 1;
      }
    }
    return this.mt[0] = 0x80000000;
  };

  MersenneTwister.prototype.nextInteger = function(upper) {
    var kk, mag01, y;
    if ((upper != null ? upper : 1) < 1) {
      return 0;
    }
    mag01 = [0, this.MATRIX_A];
    if (this.mti >= this.N) {
      kk = 0;
      while (kk < this.N - this.M) {
        y = this.unsigned32((this.mt[kk] & this.UPPER_MASK) | (this.mt[kk + 1] & this.LOWER_MASK));
        this.mt[kk] = this.unsigned32(this.mt[kk + this.M] ^ (y >>> 1) ^ mag01[y & 0x1]);
        kk++;
      }
      while (kk < this.N - 1) {
        y = this.unsigned32((this.mt[kk] & this.UPPER_MASK) | (this.mt[kk + 1] & this.LOWER_MASK));
        this.mt[kk] = this.unsigned32(this.mt[kk + this.M - this.N] ^ (y >>> 1) ^ mag01[y & 0x1]);
        kk++;
      }
      y = this.unsigned32((this.mt[this.N - 1] & this.UPPER_MASK) | (this.mt[0] & this.LOWER_MASK));
      this.mt[this.N - 1] = this.unsigned32(this.mt[this.M - 1] ^ (y >>> 1) ^ mag01[y & 0x1]);
      this.mti = 0;
    }
    y = this.mt[this.mti++];
    y = this.unsigned32(y ^ (y >>> 11));
    y = this.unsigned32(y ^ ((y << 7) & 0x9d2c5680));
    y = this.unsigned32(y ^ ((y << 15) & 0xefc60000));
    return this.unsigned32(y ^ (y >>> 18)) % (upper != null ? upper : 0x100000000);
  };

  MersenneTwister.prototype.nextFloat = function() {
    return this.nextInteger() / 0xffffffff;
  };

  MersenneTwister.prototype.nextBoolean = function() {
    return this.nextInteger() % 2 === 0;
  };

  return MersenneTwister;

})();

Maze.Algorithms.RecursiveBacktracker = (function(_super) {

  __extends(RecursiveBacktracker, _super);

  RecursiveBacktracker.prototype.IN = 0x1000;

  RecursiveBacktracker.prototype.STACK = 0x2000;

  RecursiveBacktracker.prototype.START = 1;

  RecursiveBacktracker.prototype.RUN = 2;

  RecursiveBacktracker.prototype.DONE = 3;

  function RecursiveBacktracker(maze, options) {
    RecursiveBacktracker.__super__.constructor.apply(this, arguments);
    this.state = this.START;
    this.stack = [];
  }

  RecursiveBacktracker.prototype.step = function() {
    switch (this.state) {
      case this.START:
        this.startStep();
        break;
      case this.RUN:
        this.runStep();
    }
    return this.state !== this.DONE;
  };

  RecursiveBacktracker.prototype.startStep = function() {
    var x, y, _ref;
    _ref = [this.rand.nextInteger(this.maze.width), this.rand.nextInteger(this.maze.height)], x = _ref[0], y = _ref[1];
    this.maze.carve(x, y, this.IN | this.STACK);
    this.updateAt(x, y);
    this.stack.push({
      x: x,
      y: y,
      dirs: this.rand.randomDirections()
    });
    this.state = this.RUN;
    return this.carvedOnLastStep = true;
  };

  RecursiveBacktracker.prototype.runStep = function() {
    var current, dir, nx, ny,
      _this = this;
    while (true) {
      current = this.stack[this.stack.length - 1];
      dir = current.dirs.pop();
      nx = current.x + Maze.Direction.dx[dir];
      ny = current.y + Maze.Direction.dy[dir];
      if (this.maze.isValid(nx, ny)) {
        if (this.maze.isBlank(nx, ny)) {
          this.stack.push({
            x: nx,
            y: ny,
            dirs: this.rand.randomDirections()
          });
          this.maze.carve(current.x, current.y, dir);
          this.updateAt(current.x, current.y);
          this.maze.carve(nx, ny, Maze.Direction.opposite[dir] | this.STACK);
          this.updateAt(nx, ny);
          if (!this.carvedOnLastStep) {
            this.eventAt(nx, ny);
          }
          this.carvedOnLastStep = true;
          break;
        } else if (this.canWeave(dir, nx, ny)) {
          this.performWeave(dir, current.x, current.y, function(x, y) {
            _this.stack.push({
              x: x,
              y: y,
              dirs: _this.rand.randomDirections()
            });
            if (!_this.carvedOnLastStep) {
              _this.eventAt(x, y);
            }
            return _this.maze.carve(x, y, _this.STACK);
          });
          this.carvedOnLastStep = true;
          break;
        }
      }
      if (current.dirs.length === 0) {
        this.maze.uncarve(current.x, current.y, this.STACK);
        this.updateAt(current.x, current.y);
        if (this.carvedOnLastStep) {
          this.eventAt(current.x, current.y);
        }
        this.stack.pop();
        this.carvedOnLastStep = false;
        break;
      }
    }
    if (this.stack.length === 0) {
      return this.state = this.DONE;
    }
  };

  RecursiveBacktracker.prototype.isStack = function(x, y) {
    return this.maze.isSet(x, y, this.STACK);
  };

  return RecursiveBacktracker;

})(Maze.Algorithm);

function genMaze(w, h, seed) {
  if(w%2 == 0 || h%2 == 0){
    throw "Only odd number of columns.";
  } else {
    var res = _.map(_.range(h), function () { return _.map(_.range(w), function () { return 1 }) })
    m = new Maze((w-1)/2, (h-1)/2, Maze.Algorithms.RecursiveBacktracker, {seed: seed});
    m.generate();
    for (var i = 1; i < w; i += 2 ) {
      for (var j = 1; j < h; j += 2 ) {
        res[j][i] = 0;
        var x = (i - 1) / 2;
        var y = (j - 1) / 2;
        if (m.isWest(x,y)) {
          res[j][i-1] = 0;
        }
        if (m.isSouth(x,y)) {
          res[j+1][i] = 0;
        }
        if (m.isNorth(x,y)) {
          res[j-1][i] = 0;
        }
        if (m.isEast(x,y)) {
          res[j][i+1] = 0;
        }
      }
    }
    return res;
  }
}
