codes = {
  37: "left",
  38: "up",
  39: "right",
  40: "down"
}

config = {
  tilesize: 120,
  width: null,
  height: null,
  difficulty: "easy"
}

ig.module( 
	"game.main" 
)
.requires(
	"impact.game",
	"game.player",
	"game.items",
	"game.maze"
)
.defines(function(){
  Loader = ig.Loader.extend({
    mickeyHead: new ig.Image("media/mickey_head.png"),

    draw: function(){
      var w = ig.system.width;
      var h = ig.system.height;

      ig.system.context.fillStyle = '#333';
      ig.system.context.fillRect(0, 0, w, h);
      this.mickeyHead.draw(w/2-this.mickeyHead.width/2-1, h/2-this.mickeyHead.height/2-13);

      var percentage = this.status * 100 >> 0;
      ig.system.context.font = "22px disneyjunior-bold";
      ig.system.context.fillStyle = '#ffffff';
      ig.system.context.textAlign = "center";
      ig.system.context.fillText(percentage, w/2, h/2);
    },

    end: function(){
      $("#canvas").trigger("loaded");
      this.parent();
    }
  });

  ig.Input.inject({
    bindTouch: function(selector, action) {
      var element = ig.$( selector );
      var that = this;
      element.addEventListener('mousedown', function(ev) {that.touchStart( ev, action );}, false);
      element.addEventListener('mouseup', function(ev) {that.touchEnd( ev, action );}, false);
      this.parent(selector, action);
    },
  });

  GameLaberinto = ig.Game.extend({

    clearColor: "#61a33e",
    paused: true,
    backgroundTiles: new ig.Image("media/tiles.png"),
    musicFile: new ig.Sound("media/sounds/music.*", false),
    buzz: new ig.Sound("media/sounds/buzz.*", false),
    fanfare: new ig.Sound("media/sounds/fanfare.*", false),

    init: function(){
      // map
      var maze = new IGMaze(config.width, config.height);
      var map = new ig.BackgroundMap(config.tilesize, maze.backgroundMap, this.backgroundTiles );      
      this.backgroundMaps.push(map);
      this.collisionMap = new ig.CollisionMap(config.tilesize, maze.collisionMap);

      this.spawnEntity(EntityArrow, config.tilesize*(config.width-3)+10, config.tilesize*(config.height-2)+25, {})

      var playerSettings = {map: maze.collisionMap, end: {x: config.width-1, y: config.height-2}};
      this.player = this.spawnEntity(EntityPlayer, config.tilesize+config.tilesize/3-20, config.tilesize+config.tilesize/3, playerSettings)

      this.spawnEntity(EntityStart, 0, config.tilesize*1-50, {})
      this.house = this.spawnEntity(EntityEnd, config.tilesize*config.width-177, config.tilesize*(config.height-3), {})

      // camera
      this.topVertical = this.collisionMap.pxHeight - ig.system.height;
      this.topHorizontal = this.collisionMap.pxWidth - ig.system.width;

      // music
      ig.music.add(this.musicFile);
      ig.music.play();

      // input
      ig.input.bind(ig.KEY.UP_ARROW, "up");
      ig.input.bindTouch("#up", "up");
      ig.input.bind(ig.KEY.RIGHT_ARROW, "right");
      ig.input.bindTouch("#right", "right");
      ig.input.bind(ig.KEY.DOWN_ARROW, "down");
      ig.input.bindTouch("#down", "down");
      ig.input.bind(ig.KEY.LEFT_ARROW, "left");
      ig.input.bindTouch("#left", "left");
    },

    updateCamera: function(){
      var cornerX = this.player.pos.x - ig.system.width/2;
      var cornerY = this.player.pos.y - ig.system.height/2;

      if(cornerX < 0)
        cornerX = 0;
      else if(cornerX > this.topHorizontal)
        cornerX = this.topHorizontal;
      if(cornerY < 0)
        cornerY = 0;
      else if(cornerY > this.topVertical)
        cornerY = this.topVertical;

      this.screen.x = cornerX;
      this.screen.y = cornerY;
    },

    update: function(){
      if(this.paused) return;

      this.updateCamera();
      this.parent();
    },

    draw: function(){
      this.parent();
      if(this.paused){
        ig.system.context.fillStyle = "rgba(0,0,0,0.5)";
        ig.system.context.fillRect(0, 0, ig.system.width, ig.system.height);
      };
    }
  });

  if(ig.ua.mobile){
    ig.Sound.enabled = false;
  }
});
